###########################################################
### The lifespan development of cognate guessing skills ###
### in an unknown language                              ###
### Jan Vanhove & Raphael Berthele			###
### (jan.vanhove@unifr.ch)				###
###########################################################

###############################
# Set up graphical parameters #
###############################

# Load necessary packages
library(ggplot2)

# Settings ggplot2
theme_set(theme_bw())
theme_update(axis.title = element_text(size=10))
GeomSmooth <- ggplot2:::GeomSmooth

GeomSmooth$default_aes <- function (.)
  aes(colour = "black", fill = "grey70", size = 0.5, linetype = 1,
      weight = 1, alpha = 0.4)

environment(GeomSmooth) <- environment(ggplot2:::GeomSmooth)
assignInNamespace("GeomSmooth", GeomSmooth, ns="ggplot2")

# Multiple plot function
#
# ggplot objects can be passed in ..., or to plotlist (as a list of ggplot objects)
# - cols:   Number of columns in layout
# - layout: A matrix specifying the layout. If present, 'cols' is ignored.
#
# If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
# then plot 1 will go in the upper left, 2 will go in the upper right, and
# 3 will go all the way across the bottom.
#
multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  require(grid)
  
  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)
  
  numPlots = length(plots)
  
  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols),
                     byrow=TRUE)
  }
  
  if (numPlots==1) {
    print(plots[[1]])
    
  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
    
    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))
      
      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}

###############
# Preparation #
###############

# Packages
library(lme4)
library(mgcv)
library(plyr)
library(languageR)

# Read in data
dat <- read.csv("cognate_guessing.csv", header=T)
dat$Correct <- factor(dat$Correct)
dat$Subject <- factor(dat$Subject)
nrow(dat) # 16,300 data points

# Remove incomplete data
dat = subset(dat, !is.na(WST) & !is.na(English.MC) & !is.na(English.Cloze))
nrow(dat) # 15,900 data points

####################
# Data description #
####################

### TABLE 1: Age groups and proportion of each sex
dat$AgeGroup = cut(dat$Age, breaks=c(9, 13, 17, 29, 39, 49, 59, 69, 79, 100))
ageGroups = ddply(dat, .(AgeGroup), summarise,
                  Men = length(which(Sex=="male"))/100,
                  Women = length(which(Sex=="female"))/100,
                  MinAge = min(Age),
                  MaxAge = max(Age),
                  MeanAge = mean(Age))
ageGroups

### TABLE 2: Number of correctly translated words per modality
perSubject <- ddply(dat, .(Subject), summarise,
                    Spoken = length(which(Correct=="1" & Status=="target" & Modality=="Auditory")),
                    Written = length(which(Correct=="1" & Status=="target" & Modality=="Visual")),
                    SpokenProfile = length(which(Correct=="1" & Status=="profile" & Modality=="Auditory")),
                    WrittenProfile = length(which(Correct=="1" & Status=="profile" & Modality=="Visual")))
summary(perSubject)

# Create indicator variable
perSubject$ProfileRight <- factor(perSubject$SpokenProfile > 0 | perSubject$WrittenProfile > 0)
# Merge
dat = merge(dat, perSubject, by="Subject")

# Correct per item
perItem <- ddply(dat, .(Stimulus), summarise,
                 Correct = length(which(Correct=="1")),
                 Modality = unique(Modality),
                 Status = unique(Status))
perItem[perItem$Status == "profile",]
perItem[perItem$Status=="target" & perItem$Correct == 0, ]

### TABLE 3
# Only 6 participants with more than 5 foreign languages
nrow(dat[dat$NrLang > 5,])/100
# Collapse into category of 5 languages
dat[dat$NrLang >= 5,]$NrLang <- 5

# Computer overall English proficiency measure
# by averaging z-scores of English.MC and English.Cloze
with(dat, cor(English.MC, English.Cloze))
z. = function(x) (x-mean(x))/sd(x)
dat$English.Overall = rowMeans(cbind(z.(dat$English.MC), z.(dat$English.Cloze)))
with(dat, cor(English.MC, English.Overall)); with(dat, cor(English.Cloze, English.Overall))

# Summary tables
summary(unique(dat[,c("Subject", "NrLang", "English.MC", "English.Cloze", "English.Overall",
                      "WST", "Raven", "BWDS")]))
sapply(unique(dat[,c("Subject", "NrLang", "English.MC", "English.Cloze", "English.Overall",
                     "WST", "Raven", "BWDS")]), sd)[-1]

# Multicollinearity assessment
c. = function(x) x-mean(x)
dat$cNrLang = c.(dat$NrLang)
dat$cEnglish.Overall = c.(dat$English.Overall)
dat$cWST = c.(dat$WST)
dat$cRaven = c.(dat$Raven)
dat$cBWDS = c.(dat$BWDS)
cor(dat[,c("cNrLang", "cEnglish.Overall",
                  "cWST", "cRaven", "cBWDS")])
collin.fnc(dat[,c("cNrLang", "cEnglish.Overall",
                         "cWST", "cRaven", "cBWDS")])$cnumber

# Lifespan trends
# Summary per participant
perSubject <- ddply(dat, .(Subject), summarise,
                    Age = unique(Age),
                    NrLang = unique(NrLang),
                    English.Overall = unique(English.Overall),
                    WST = unique(WST),
                    Raven = unique(Raven),
                    BWDS = unique(BWDS))

# Lifespan trends per variable
lifespan_NrLang <- ggplot(perSubject, aes(Age, NrLang)) +
  geom_point() +
  stat_smooth(method="gam", formula=y~s(x, bs="ad")) + # adaptive smoother (doesn't assume homogeneously 'wiggly' trends)
  xlab("Age (years)") +
  ylab("Number of foreign languages")

lifespan_English <- ggplot(perSubject, aes(Age, English.Overall)) +
  geom_point() +
  stat_smooth(method="gam", formula=y~s(x, bs="ad")) +
  xlab("Age (years)") +
  ylab("English proficiency")

lifespan_WST <- ggplot(perSubject, aes(Age, WST)) +
  geom_point() +
  stat_smooth(method="gam", formula=y~s(x, bs="ad")) +
  ylim(0, 42) +
  xlab("Age (years)") +
  ylab("WST score")

lifespan_Raven <- ggplot(perSubject, aes(Age, Raven)) +
  geom_point() +
  stat_smooth(method="gam", formula=y~s(x, bs="ad")) +
  ylim(0, 36) +
  xlab("Age (years)") +
  ylab("Raven score")

lifespan_BWDS <- ggplot(perSubject, aes(Age, BWDS)) +
  geom_point() +
  stat_smooth(method="gam", formula=y~s(x, bs="ad")) +
  ylim(0, 14) +
  xlab("Age (years)") +
  ylab("Backward digit span")

# Plot lifespan
png("lifespans.png", width=6.5, height=8, units="in", res=1200)
multiplot(lifespan_NrLang, lifespan_English, lifespan_WST, lifespan_Raven, lifespan_BWDS, cols=2)
dev.off()
rm(list=c("lifespan_NrLang", "lifespan_English", "lifespan_WST", "lifespan_Raven", "lifespan_BWDS"))

#############
# GAMMs Age #
#############

# Filter data
dat = subset(dat, Stimulus != "kagel" & Stimulus != "skarm" & Stimulus != "tydlig")
dat = subset(dat, Status=="target")

# Split up according to modality
dat_written = subset(dat, Modality=="Visual")
dat_spoken = subset(dat, Modality=="Auditory")

# GAMM written
age_written = bam(Correct ~ s(Age, bs="ad") + Sex + ProfileRight + s(Subject, bs="re") + s(Stimulus, bs="re"), data=dat_written, family=binomial)
age_written2 = bam(Correct ~ s(Age, bs="ad") + ProfileRight + s(Subject, bs="re") + s(Stimulus, bs="re"), data=dat_written, family=binomial)

# GAMM spoken
age_spoken = bam(Correct ~ s(Age, bs="ad") + Sex + ProfileRight + s(Subject, bs="re") + s(Stimulus, bs="re"), data=dat_spoken, family=binomial)
age_spoken2 = bam(Correct ~ s(Age, bs="ad") + s(Subject, bs="re") + s(Stimulus, bs="re"), data=dat_spoken, family=binomial)

rm(age_written2, age_spoken2)
# Plot
#png("gamms.png", width=5, height=7, units="in", res=1200)
#par(mfrow=c(2,1), mar=c(4,4,2,0.5)+0.1, cex=0.7)
png("gamms.png", width=6.5, height=3.5, units="in", res=1200)
par(mfrow=c(1,2), mar=c(4,4,1.5,0.5)+0.1, las=1, cex=0.7)
plot.gam(age_written, select=1, shade=T, trans=plogis, shift=-0.7375,
         xlab="Age (years)",
         ylab="Probability of correct translation", ylim=c(-5,0.4054651)+0.7375,
         main="Written target words",
         las=1)
plot.gam(age_spoken, select=1, shade=T, trans=plogis, shift=-1.04199,
         xlab="Age (years)",
         ylab="Probability of correct translation", ylim=c(-5,0.4054651)+1.04199,
         main="Spoken target words",
         las=1)
par(mfrow=c(1,1))
dev.off()

age_nointeraction = bam(Correct ~ Modality +  s(Age, bs="ad") + Modality*Sex + Modality*ProfileRight + s(Subject, bs="re") + s(Stimulus, bs="re"), data=dat, family=binomial)
age_interaction = bam(Correct ~ Modality + s(Age, bs="ad", by=Modality) + Modality*Sex + Modality*ProfileRight + s(Subject, bs="re") + s(Stimulus, bs="re"), data=dat, family=binomial)
AIC(age_nointeraction)-AIC(age_interaction)

rm(age_written, age_spoken, age_nointeraction, age_interaction)

#######################
# Cognitive variables #
#######################

## Written
# Exploration nonlinear trends:
bam_written = bam(Correct ~ Sex + ProfileRight + NrLang + s(English.Overall) + s(WST) + s(Raven) + s(BWDS) + s(Subject, bs="re") + s(Stimulus, bs="re"), data=dat_written, family=binomial)
rm(bam_written)

# Final written model
dat_written$cNrLang = c.(dat_written$NrLang)
dat_written$cEnglish.Overall = c.(dat_written$English.Overall)
dat_written$cWST = c.(dat_written$WST)
dat_written$cRaven = c.(dat_written$Raven)
dat_written$cBWDS = c.(dat_written$BWDS)
mod_written = glmer(Correct ~ Sex + ProfileRight + 
                             cNrLang + cEnglish.Overall +
                             cWST + cRaven + cBWDS +
                             (1|Subject) +
                             (1+cEnglish.Overall+cWST|Stimulus) + (0+cRaven|Stimulus),
                   data = dat_written, family=binomial)

# Without nonsign
mod_written2 = glmer(Correct ~ ProfileRight + 
                     cNrLang + cEnglish.Overall +
                     cWST +
                     (1|Subject) +
                     (1+cEnglish.Overall+cWST|Stimulus),
                   data = dat_written, family=binomial)
rm(mod_written2)

## Spoken
# Exploration nonlinear trends
bam_spoken = bam(Correct ~ Sex + ProfileRight + NrLang + s(English.Overall) + s(WST) + s(Raven) + s(BWDS) + s(Subject, bs="re") + s(Stimulus, bs="re"), data=dat_spoken, family=binomial)

# Final model
dat_spoken$cNrLang = c.(dat_spoken$NrLang)
dat_spoken$cEnglish.Overall = c.(dat_spoken$English.Overall)
dat_spoken$cWST = c.(dat_spoken$WST)
dat_spoken$cRaven = c.(dat_spoken$Raven)
dat_spoken$cBWDS = c.(dat_spoken$BWDS)
mod_spoken = glmer(Correct ~ Sex + ProfileRight + 
                     cNrLang + cEnglish.Overall +
                     cWST + cRaven + cBWDS +
                     (1|Subject) +
                     (1+cEnglish.Overall+cWST|Stimulus) + (0+cRaven|Stimulus),
                   data = dat_spoken, family=binomial)
mod_spoken2 = glmer(Correct ~ Sex +
                     cEnglish.Overall +
                     cWST + cRaven + cDS.Total +
                     (1|Subject) +
                     (1+cEnglish.Overall+cWST|Stimulus) + (0+cRaven|Stimulus),
                   data = dat_spoken, family=binomial)
rm(mod_spoken2)

## Plot partial effect of each variable
png("lmer_written.png", width=6.5, height=6.5, units="in", res=1200)
par(mfrow=c(3,3), mar=c(4,4,1,0.5)+0.1, oma=c(0,0,2,0), las=1, cex=0.7)
plotLMER.fnc(mod_written, ylim=c(0, 0.6), cex=0.6, las=1, ylabel="Probability of correct translation",
             pred="Sex",
             xlabel="Sex")
plotLMER.fnc(mod_written, ylim=c(0, 0.6), cex=0.6, las=1, ylabel="Probability of correct translation",
             pred="ProfileRight",
             xlabel="More than one correct profile\nword translation?")
plotLMER.fnc(mod_written, xlabel="Number of foreign languages",
             pred="cNrLang",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=1:5-mean(dat_written$NrLang),
     labels=as.character(1:5))
plotLMER.fnc(mod_written, xlabel="English proficiency",
             pred="cEnglish.Overall",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=seq(-2.5,1.5,1)-mean(dat_written$English.Overall),
     labels=as.character(seq(-2.5,1.5,1)))
plotLMER.fnc(mod_written, xlabel="WST score",
             pred="cWST",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=seq(0,42,10)-mean(dat_written$WST),
     labels=as.character(seq(0,42,10)))
plotLMER.fnc(mod_written, xlabel="Raven score",
             pred="cRaven",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=seq(0,36,10)-mean(dat_written$Raven),
     labels=as.character(seq(0,36,10)))
plotLMER.fnc(mod_written, xlabel="Backward digit span",
             pred="cBWDS",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=seq(2,12,2)-mean(dat_written$BWDS),
     labels=as.character(seq(2,12,2)))
title("Written target words", outer=TRUE)
dev.off()

png("lmer_spoken.png", width=6.5, height=6.5, units="in", res=1200)
par(mfrow=c(3,3), mar=c(4,4,1,0.5)+0.1, oma=c(0,0,2,0), las=1, cex=0.7)
plotLMER.fnc(mod_spoken, ylim=c(0, 0.6), cex=0.6, las=1, ylabel="Probability of correct translation",
             pred="Sex",
             xlabel="Sex")
plotLMER.fnc(mod_spoken, ylim=c(0, 0.6), cex=0.6, las=1, ylabel="Probability of correct translation",
             pred="ProfileRight",
             xlabel="More than one correct profile\nword translation?")
plotLMER.fnc(mod_spoken, xlabel="Number of foreign languages",
             pred="cNrLang",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=1:5-mean(dat_spoken$NrLang),
     labels=as.character(1:5))
plotLMER.fnc(mod_spoken, xlabel="English proficiency",
             pred="cEnglish.Overall",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=seq(-2.5,1.5,1)-mean(dat_spoken$English.Overall),
     labels=as.character(seq(-2.5,1.5,1)))
plotLMER.fnc(mod_spoken, xlabel="WST score",
             pred="cWST",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=seq(0,42,10)-mean(dat_spoken$WST),
     labels=as.character(seq(0,42,10)))
plotLMER.fnc(mod_spoken, xlabel="Raven score",
             pred="cRaven",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=seq(0,36,10)-mean(dat_spoken$Raven),
     labels=as.character(seq(0,36,10)))
plotLMER.fnc(mod_spoken, xlabel="Backward digit span",
             pred="cBWDS",
             las=1, cex=0.6, xaxt="n",
             ylabel="Probability of correct translation",
             ylimit=c(0,0.6))
axis(1, at=seq(2,12,2)-mean(dat_spoken$BWDS),
     labels=as.character(seq(2,12,2)))
title("Spoken target words", outer=TRUE)
dev.off()


## Interactions
# Fair warning: this takes ages unless you're working at CERN.
mod_all <- glmer(Correct ~ Modality*Sex + Modality*ProfileRight +
                     Modality*cNrLang + Modality*cEnglish.Overall +
                     Modality*cWST + Modality*cRaven + Modality*cBWDS +
                     (1|Subject) + (0+Modality|Subject) +
                     (1+cEnglish.Overall+cWST|Stimulus) + (0+cRaven|Stimulus),
                   data = dat, family=binomial)

## Random intercepts (per subject)
ranef_subj_written = data.frame(ranef(mod_written)[["Subject"]])
ranef_subj_written$Subject = factor(rownames(ranef_subj_written))
colnames(ranef_subj_written) = c("RE_Written", "Subject")

ranef_subj_spoken = data.frame(ranef(mod_spoken)[["Subject"]])
ranef_subj_spoken$Subject = factor(rownames(ranef_subj_spoken))
colnames(ranef_subj_spoken) = c("RE_Spoken", "Subject")

perSubject = merge(perSubject, ranef_subj_written, by="Subject", all.y=T)
perSubject = merge(perSubject, ranef_subj_spoken, by="Subject", all.y=T)

# Can any variance in REs be explained by age?
gam_ranef_written = gam(RE_Written ~ s(Age, bs="ad"), data = perSubject)
gam_ranef_spoken = gam(RE_Spoken ~ s(Age, bs="ad"), data = perSubject)

png("residual_age.png", width=6.5, height=3.5, units="in", res=1200)
par(mfrow=c(1,2), mar=c(4,4,1.5,0.5)+0.1, las=1, cex=0.7)
plot(gam_ranef_written, shade=T, ylim=c(-1, 0.5), xlab="Age (years)", ylab="Random intercept",
     main="Written target words")
plot(gam_ranef_spoken, shade=T, ylim=c(-1, 0.5), xlab="Age (years)", ylab="Random intercept",
     main="Spoken target words")
dev.off()
